@@ -162,10 +162,11 @@ module Database.Redis (
     RedisCtx(..), MonadRedis(..),
 
     -- * Connection
-    Connection, ConnectError(..), connect, checkedConnect, disconnect, 
+    Connection, ConnectError(..), connect, checkedConnect, disconnect,
+    withConnect, withCheckedConnect,
     ConnectInfo(..), defaultConnectInfo, parseConnectInfo,
-    HostName, PortID(..),
-    
+    PortID(..),
+
     -- * Commands
     module Database.Redis.Commands,
     
@@ -195,7 +196,7 @@ import Database.Redis.Core
 import Database.Redis.PubSub
 import Database.Redis.Protocol
 import Database.Redis.ProtocolPipelining
-    (HostName, PortID(..), ConnectionLostException(..))
+    (PortID(..), ConnectionLostException(..))
 import Database.Redis.Transactions
 import Database.Redis.Types
 import Database.Redis.URL
