@@ -1,4 +1,7 @@
-{-# LANGUAGE RecordWildCards, DeriveDataTypeable, OverloadedStrings #-}
+{-# LANGUAGE DeriveDataTypeable #-}
+{-# LANGUAGE LambdaCase #-}
+{-# LANGUAGE OverloadedStrings #-}
+{-# LANGUAGE RecordWildCards #-}
 
 -- |A module for automatic, optimal protocol pipelining.
 --
@@ -17,7 +20,7 @@ module Database.Redis.ProtocolPipelining (
   Connection,
   connect, enableTLS, beginReceiving, disconnect, request, send, recv, flush,
   ConnectionLostException(..),
-  HostName, PortID(..)
+  PortID(..)
 ) where
 
 import           Prelude
@@ -31,8 +34,6 @@ import qualified Data.ByteString as S
 import qualified Data.ByteString.Lazy as L
 import           Data.IORef
 import           Data.Typeable
-import           Network
-import qualified Network.BSD as BSD
 import qualified Network.Socket as NS
 import qualified Network.TLS as TLS
 import           System.IO
@@ -41,6 +42,10 @@ import           System.IO.Unsafe
 
 import           Database.Redis.Protocol
 
+data PortID = PortNumber NS.PortNumber
+            | UnixSocket String
+            deriving (Eq, Show)
+
 data ConnectionContext = NormalHandle Handle | TLSContext TLS.Context
 
 data Connection = Conn
@@ -71,8 +76,33 @@ data ConnectTimeout = ConnectTimeout ConnectPhase
 
 instance Exception ConnectTimeout
 
-connect :: HostName -> PortID -> Maybe Int -> IO Connection
-connect hostName portID timeoutOpt =
+getHostAddrInfo :: NS.HostName -> NS.PortNumber -> IO [NS.AddrInfo]
+getHostAddrInfo hostname port = do
+  NS.getAddrInfo (Just hints) (Just hostname) (Just $ show port)
+  where
+    hints = NS.defaultHints
+      { NS.addrSocketType = NS.Stream }
+
+connectSocket :: [NS.AddrInfo] -> IO NS.Socket
+connectSocket [] = error "connectSocket: unexpected empty list"
+connectSocket (addr:rest) = tryConnect >>= \case
+  Right sock -> return sock
+  Left err   -> if null rest
+                then throwIO err
+                else connectSocket rest
+  where
+    tryConnect :: IO (Either IOError NS.Socket)
+    tryConnect = bracketOnError createSock NS.close $ \sock -> do
+      try (NS.connect sock $ NS.addrAddress addr) >>= \case
+        Right () -> return (Right sock)
+        Left err -> NS.close sock >> return (Left err)
+      where
+        createSock = NS.socket (NS.addrFamily addr)
+                               (NS.addrSocketType addr)
+                               (NS.addrProtocol addr)
+
+connect :: NS.HostName -> PortID -> Maybe Int -> IO Connection
+connect hostName portId timeoutOpt =
   bracketOnError hConnect hClose $ \h -> do
     hSetBinaryMode h True
     connReplies <- newIORef []
@@ -83,7 +113,7 @@ connect hostName portID timeoutOpt =
   where
         hConnect = do
           phaseMVar <- newMVar PhaseUnknown
-          let doConnect = hConnect' portID phaseMVar
+          let doConnect = hConnect' phaseMVar
           case timeoutOpt of
             Nothing -> doConnect
             Just micros -> do
@@ -93,16 +123,20 @@ connect hostName portID timeoutOpt =
                 Right () -> do
                   phase <- readMVar phaseMVar
                   errConnectTimeout phase
-        hConnect' (PortNumber port) mvar =
-          bracketOnError mkSocket NS.close $ \sock -> do
-            NS.setSocketOption sock NS.KeepAlive 1
-            void $ swapMVar mvar PhaseResolve
-            host <- BSD.getHostByName hostName
-            void $ swapMVar mvar PhaseOpenSocket
-            NS.connect sock $ NS.SockAddrInet port (BSD.hostAddress host)
-            NS.socketToHandle sock ReadWriteMode
-        hConnect' _ _ = connectTo hostName portID
-        mkSocket   = NS.socket NS.AF_INET NS.Stream 0
+        hConnect' mvar = bracketOnError createSock NS.close $ \sock -> do
+          NS.setSocketOption sock NS.KeepAlive 1
+          void $ swapMVar mvar PhaseResolve
+          void $ swapMVar mvar PhaseOpenSocket
+          NS.socketToHandle sock ReadWriteMode
+          where
+            createSock = case portId of
+              PortNumber portNumber -> do
+                addrInfo <- getHostAddrInfo hostName portNumber
+                connectSocket addrInfo
+              UnixSocket addr -> bracketOnError
+                (NS.socket NS.AF_UNIX NS.Stream NS.defaultProtocol)
+                NS.close
+                (\sock -> NS.connect sock (NS.SockAddrUnix addr) >> return sock)
 
 enableTLS :: TLS.ClientParams -> Connection -> IO Connection
 enableTLS tlsParams conn@Conn{..} = do
